#ifndef MOTOR_H__
#define MOTOR_H__

/* Updated during calibration */
extern volatile int motor_pwm_ticks;
extern volatile int motor_pwm_rpm;
extern volatile Position motor_pwm_position;

/* Emergency stop.  Immediately kills power to motor and grounds all
   phases for full braking.  */
extern void motor_estop ();

/* Graceful stop.  Disconnects power, waits for motor to slow to 500
   RPM, then brakes to a stop.  */
extern void motor_stop ();

/* Travel to absolute POSITION (reference position is encoder_lval() */
extern void motor_travel (Position);
Position calib_target_position;

/* Track step/direction input, zero referenced to where the motor was
   when this function is called. */
extern void motor_track ();

/* When it track or travel mode, this adjusts the target position
   without changing anything else.  */
extern void motor_step (Position);

/* Run at a constant RPM, or as fast as possible if the desired RPM is
   too high.  This is an approximation position-wise (i.e. it tends to
   lag) but it's only used for finding the home/limit switches.  */
extern void motor_rpm (int);

/* Run at a constant TORQUE (-32768...32768)  */
extern void motor_torque (int);

/* Do not drive the motor at all; allows the calibration routines to
   manually drive it.  In this mode, motor_pwm_ticks is incremented
   once per motor control loop.  */
extern void motor_calibrate ();

/* Likewise, but the RPM control loop runs, trying to keep the RPM at
   calib_desired_rpm.  */
extern void motor_calibrate_rpm_loop ();
extern volatile float calib_desired_rpm;

/* Likewise, but the innermost current control loop runs, trying to
   keep the current at calib_desired_current.  */
extern void motor_calibrate_current_loop ();
extern volatile float calib_desired_current;

#endif
