#include <stdio.h>
#include <math.h>
#include "bldc.h"

double
dcos (double pos)
{
  return cos (pos * M_PI / 90);
}

#define MINWAIT 300

#define NTICKS 40
static int ticks[NTICKS];
static int tidx = 0;

main()
{
  int i;
  int halls, old_halls = -1;
  int u = 0;
  int v = 0;
  int w = 0;
  int power = 0;
  int pos, old_pos = -999;
  int running = 0;
  int update_status = 1;
  int then = 0;
  int lead = -45;
  int dloop = 0;
  int odloop = 0;
  int target = 0, old_target = 0;
  int time_last_pos_change = 0;

  bldc_init ();
  encoder_init ();
  cpwm_init ();
  thermistor_init ();

  tty_puts("\033[1;1H\033[2J\n\nHello from Sine 1!\n");

  /* Shut off the motor.  */
  PORTB.DR.BYTE = UL | VL | WL;

  simple_commutate (0);

  printf("Looping...\n");

  while (1)
    {

      if (tty_ready ())
	{
	  int c = tty_getc ();
	  switch (c)
	    {
	    case 'r':
	      bldc_run ();
	      old_pos = -999;
	      running = 1;
	      break;

	    case 's':
	    case ' ':
	      bldc_stop ();
	      old_pos = -999;
	      running = 0;
	      break;

	    case '0' ... '9':
	      if (c == '0')
		power = 32700;
	      else
		power = (c - '0') * 32767/10;
	      old_pos = -999;
	      break;

	    case '-':
	    case '_':
	      target -= 5;
	      tidx = 0;
	      break;
	    case '+':
	    case '=':
	      target += 5;
	      tidx = 0;
	      break;

	    case 't':
	      printf("\033[5;1H");
	      then = ticks[0];
	      for (i=1; i<tidx; i++)
		printf("%3d %6d\n", i, (short)ticks[i] - (short)then);
	      printf("\033[J");
	      fflush (stdout);
	      break;

	    }
	}

      pos = encoder_ang ();

      if (pos != old_pos || target != old_target)
	{
	  old_pos = pos;
	  old_target = target;
	  update_status = 1;

	  if (tidx < NTICKS-1)
	    ticks[tidx++] = msec;
	  time_last_pos_change = msec;

	  cpwm_levels (power * dcos(target -  70),
		       power * dcos(target - 130),
		       power * dcos(target - 190));
	}
      else
	{
	  if (msec - time_last_pos_change > 30)
	    {
	      cpwm_levels (0, 0, 0);
	    }
	}

      if (tty_idle () && (msec - then) > MINWAIT)
	{
	  then = msec;
	  update_status = 0;
	  printf("\033[1;1H  pos = %5d vs %5d, \033[%dmpower = %5d\033[0m  ",
		 pos, target, running ? 32 : 31, power);

	  temperature();

	  printf("\033[K");
	  fflush (stdout);
	}
    }

  exit (0);
}
