#include <stdio.h>
#include "bldc.h"

#define U0 0
#define V0 0
#define W0 0

struct {
  unsigned char halls;
  unsigned char drives;
} phase[] = {
  { U_HALL          | W_HALL, UH | VL | W0 },
  { U_HALL,                   UH | V0 | WL },
  { U_HALL | V_HALL,          U0 | VH | WL },
  {          V_HALL,          UL | VH | W0 },
  {          V_HALL | W_HALL, UL | V0 | WH },
  {                   W_HALL, U0 | VL | WH },
};

unsigned char commutations[16];

static char *str (int v, int b)
{
  static char buf[3][10];
  static int bufp = 0;
  char *bp;

  bufp = (bufp+1) % 3;
  bp = buf[bufp];
  while (b)
    {
      *bp++ = (v & (1<<(b-1))) ? 'H' : '_';
      b--;
    }
  *bp++ = 0;
  return buf[bufp];
}

main()
{
  int then = 0;
  int i;
  int old_halls = -1;
  int run;

  bldc_init ();
  encoder_init ();

  tty_puts("Hello from Trap 1!\n");
  tty_puts("Simple trapezoid control using hall sensors in software\n");

  for (i=0; i<16; i++)
    commutations[i] = UL | VL | WL;

  for (i=0; i<6; i++)
    commutations[phase[i].halls] = phase[i].drives;

  run = 0;
  printf("Stopped\n");

  i = 0;
  while (1)
    {
      int halls = HALLS & (U_HALL|V_HALL|W_HALL);

      if (tty_ready ())
	{
	  int c = tty_getc ();
	  switch (c)
	    {
	    case 'r':
	      printf("Running\n");
	      run = 1;
	      break;
	    case 's':
	    case ' ':
	      printf("Stop\n");
	      run = 0;
	      break;
	    }
	}

      if (run)
	PORT7.DR.BYTE = commutations[halls];
      else
	PORT7.DR.BYTE = UL | VL | WL;

#if 1
#define MINWAIT 500
      if (tty_idle () && (msec - then) > MINWAIT)
	{
	  then = msec;
	  printf("%6d \r", encoder_rpm ());
	  fflush (stdout);
	}
#endif

#if 0
      if (old_halls != halls)
	{
	  old_halls = halls;
	  printf("  HALL: %c %c %c  PHASE: %s %s %s \033[0m\n",
		 halls & U_HALL ? 'U' : '-',
		 halls & V_HALL ? 'V' : '-',
		 halls & W_HALL ? 'W' : '-',
		 commutations[halls] & UH ? "\033[31mUH" : commutations[halls] & UL ? "\033[32mul" : "\033[34m--",
		 commutations[halls] & VH ? "\033[31mVH" : commutations[halls] & VL ? "\033[32mvl" : "\033[34m--",
		 commutations[halls] & WH ? "\033[31mWH" : commutations[halls] & WL ? "\033[32mwl" : "\033[34m--");
	  fflush (stdout);
	}
#endif

    }

  exit (0);
}
