#include <stdio.h>
#include "bldc.h"

#define DEAD 1
#define CYCLE 4096
#define MAXPOWER (CYCLE/2)

main()
{
  bldc_init ();

  tty_puts("Hello from Trap 2!\n");
  tty_puts("Simple trapezoid control using hall sensors in hardware\n");

  /* Figure 17.38 "Example of Complementary PWM Mode (Unit 0) Setting
     Procedure" */

  /* [1] Clear bits CST3 and CST4 in the timer start register (TSTRA)
         to 0 to stop timer counter (TCNT) operation. Specify
         complementary PWM mode while MTU3.TCNT and MTU4.TCNT are
         stopped. */

  MTUB.TSTR.BIT.CST3 = 0;
  MTUB.TSTR.BIT.CST4 = 0;

  /* [2] Set the same counter clock and clock edge for channels 3 and
         4 with bits TPSC[2:0] and bits CKEG[1:0] in the timer control
         register (TCR). Use bits CCLR[2:0] to set synchronous
         clearing only when restarting by synchronous clearing with
         another channel during complementary PWM mode operation. */

  MTU9.TCR.BIT.TPSC = 0;	/* PCLK/1 */
  MTU9.TCR.BIT.CKEG = 0;
  MTU9.TCR.BIT.CCLR = 3;	/* Sync */

  MTU10.TCR.BIT.TPSC = 0;	/* PCLK/1 */
  MTU10.TCR.BIT.CKEG = 0;
  MTU10.TCR.BIT.CCLR = 3;	/* Sync */

  /* [3] When performing brushless DC motor control, set bit BDC in
         the timer gate control register (TGCRA) and set the feedback
         signal input source and output chopping or gate signal direct
         output. */

  MTUB.TGCR.BIT.BCD = 0;
  MTUB.TGCR.BIT.P = 1;
  MTUB.TGCR.BIT.N = 1;
  MTUB.TGCR.BIT.FB = 0;

  /* [4] Set the dead time in MTU3.TCNT. Set MTU4.TCNT to 0000h. */

  MTU9.TCNT = DEAD;
  MTU10.TCNT = 0;

  /* [5] Set only when restarting by synchronous clearing with another
         channel during complementary PWM mode operation. In this
         case, synchronize the channel generating the synchronous
         clear with channels 3 and 4 using the timer synchronous
         register (TSYRA). */

  MTUB.TSYR.BIT.SYNC3 = 0;
  MTUB.TSYR.BIT.SYNC4 = 0;

  /* [6] Set the output PWM duty cycles in the duty registers
         (MTU3.TGRB, MTU4.TGRA, and MTU4.TGRB) and buffer registers
         (MTU3.  TGRD, MTU4.TGRC, and MTU4.TGRD). Set the same value
         in the buffer registers and the duty registers. */

  MTU9.TGRB = 1;	/* Phase 1 */
  MTU9.TGRD = 178;	/* Phase 1 buffer */
  MTU10.TGRA = 1;	/* Phase 2 */
  MTU10.TGRC = 745;	/* Phase 2 buffer */
  MTU10.TGRB = 1;	/* Phase 3 */
  MTU10.TGRD = 1067;	/* Phase 3 buffer */

  /* [7] This setting is necessary only when no dead time should be
         generated. Make appropriate settings in the timer dead time
         enable register (TDER) so that no dead time is generated. */

  MTUB.TDER.BYTE = 0;

  /* [8] Set the dead time in the dead time register (TDDRA), 1/2 the
         carrier cycle in the carrier cycle data register (TCDRA) and
         carrier cycle buffer register (TCBRA), and 1/2 the carrier
         cycle plus the dead time in MTU3.TGRA and MTU3.TGRC. When no
         dead time generation is selected, set 1 in TDDRA and 1/2 the
         carrier cycle + 1 in MTU3.TGRA and MTU3.TGRC. */

  MTUB.TDDR = DEAD;
  MTUB.TCDR = CYCLE/2;
  MTUB.TCBR = CYCLE/2;
  MTU9.TGRA = CYCLE/2 + DEAD;
  MTU9.TGRC = CYCLE/2 + DEAD;

  /* [9] Enable or disable toggle output synchronized with the PWM
         cycle using bit PSYE in the timer output control register 1
         (TOCR1A), and set the PWM output level with bits OLSP and
         OLSN. When specifying the PWM output level by using TOLBR as
         a buffer for MTU2.TOCR, see figure 17.3. */

  MTUB.TOCR1.BIT.PSYE = 1;
  MTUB.TOCR1.BIT.OLSP = 1;
  MTUB.TOCR1.BIT.OLSN = 0;
  MTUB.TOCR1.BIT.TOCS = 1;
  MTUB.TOCR1.BIT.TOCL = 0;

  MTUB.TOCR2.BYTE = 0x3f;

  /* [10] Select complementary PWM mode in the timer mode register
          (MTU3.TMDR). Set the BFA bit in MTU3.TMDR to 1 to use TGRA
          and TGRC together for buffer operation, and the BFB bit in
          MTU3.TMDR to 1 to use TGRB and TGRD together for buffer
          operation, if necessary. Set the initial Value in
          MTU4.TMDR.  */

  MTU9.TMDR.BYTE = 15; /* transfer at both */

  /* [11] Set enabling/disabling of PWM waveform output pin output in
          the timer output master enable register (TOERA). */

  MTUB.TOER.BYTE = 0x00;

  /* [12] Set the port control register and the port I/O register. */

  PORTB.DDR.BYTE = 0xff;

  /* [13] Set bits CST3 and CST4 in TSTRA to 1 simultaneously to start
          the count operation. */

  MTUB.TSTR.BYTE |= 0xc0;

  printf("Stopped\n");

  while (1)
    {
      if (tty_ready ())
	{
	  int power;
	  int c = tty_getc ();
	  switch (c)
	    {
	    case 'r':
	      printf("Running\n");
	      MTUB.TOER.BYTE = 0x3f;
	      break;

	    case 's':
	    case ' ':
	      printf("Stop\n");
	      MTUB.TOER.BYTE = 0x00;
	      break;

	    case '0' ... '9':
	      if (c == '0')
		power = MAXPOWER;
	      else
		power = (10 - (c - '0')) * MAXPOWER/10;
	      MTU9.TGRB = MTU9.TGRD = power;
	      MTU10.TGRA = MTU10.TGRC = power;
	      MTU10.TGRB = MTU10.TGRD = power;
	      printf("max power %d out of %d\n", power, MAXPOWER);
	      break;
	    }
	}
    }

  exit (0);
}
